# Copyright (C) 2008 Red Hat, Inc

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# a long with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Copyright (C) 2008 Red Hat, Inc

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# a long with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
require 'restr'
require 'yaml'
require 'xmlrpc/client'

module GenomeLib
  # This is a workaround for a Restr "feature".  Restr does not set
  # 'forcearray => true' for it's XmlSimple instance.  This means exactly
  # what you would think.  We always want to work with arrays since puppet's
  # external nodes cares what data structures are used.
  def GenomeLib.restr_get(url, xml_node=nil)
    data = xml_node ? Restr.get(url)[xml_node] : Restr.get(url)

    return case data
    when nil, Array
      data
    else
      Array.new << data
    end
  end

  module Appliances
    module Updateable
      def update_machine(uuid, config)
        raise "'uuid' must be set in #{::Genome.conf_file}" if uuid.nil?
        return Restr.post("#{@service_url}/update", {:uuid => uuid}.merge(config))
      end

      def status(info=nil)
        GenomeLib.restr_get("#{@service_url}/status.xml", info)
      end
    end

    ## A Genome appliance is a system that combines a cobbler server,
    #  a puppet master, and a collection of yaml files that define
    #  systems that have been provisioned.
    class Genome
      attr_reader :machines, :machine_types_url, :cobbler_profiles, :cobbler_profile_names, :fqdn, :repo, :git_repo_path

      def initialize(repo, git_repo_path="/srv/git") 
        @repo = repo 
        @fqdn = @repo
        @git_repo_path = git_repo_path

        @genomed = "http://#{@fqdn}/genome"
        @machine_types_url = @genomed + "/machine_types.html"
      end

      def machines
        @machines ||= fetch_data
      end

      def cobblerd
        @cobblerd ||= XMLRPC::Client.new2("http://#{@fqdn}/cobbler_api_rw")
      end

      def cobbler_call(method, *args)
        args << cobbler_token
        return cobblerd.call2(method, *args)[1]
      end

      def cobbler_profiles
        @cobbler_profiles ||= cobbler_call('get_profiles')
      end

      # Convenience method to get just the list of cobbler profile names
      def cobbler_profile_names
        @cobbler_profile_names ||= cobbler_profiles.map {|p| p["name"]}
      end

      def guest_profiles
        @guest_profiles ||= cobbler_profiles.delete_if{|p|
          p["name"].include?("__Appliance")
        }
      end

      # Deprecated
      # Use: guest_profiles instead
      def bootstrapping_cobbler_profiles
        return guest_profiles
      end

      def facts_for(type)
        # Don't want to pull in the genome lib.  See below fore more detail.
        facts = GenomeLib::restr_get("#{@genomed}/machine_types/#{type}.xml", "fact")
        return (!facts or facts.empty?) ? {} : facts.map do |f|
          def f.name
            self["name"]
          end

          def f.desc
            self["description"]  
          end

          # Here's where we make use of the DSL's simple templating
          f.instance_variable_set(:@repo, @repo)
          def f.default
            # handle there case where there is no default
            self["default"].gsub("%repo%", @repo) rescue ""
          end
          
          f
        end
      end

      def classes_for(type)
        GenomeLib::restr_get("#{@genomed}/machine_types/#{type}.xml", "class")
      end

      def fetch_data
        # Build out some mock objects so we don't have to couple to the genome
        # lib.  That lib has lots of funky metaprogramming stuff in it and it
        # might need to be changed in the future.  The fewer tools relying on it
        # the better.
        machine_types = GenomeLib::restr_get("#{@genomed}/machine_types.xml", "machine_type")

        return machine_types.map do |m|
          def m.name
            self["name"]
          end

          def m.desc
            self["description"]  
          end
           
          m
        end
      end
     
      def post_yaml(node_name, machine_type, yaml)
        Restr.post("#{@genomed}/nodes", :node_name => node_name, :yaml => yaml)
      end

      def add_system_to_cobbler(name, params, email)
        system_id = cobbler_call('new_system')
        cobbler_call('modify_system', system_id, 'name', name)
        cobbler_call('modify_system', system_id, 'profile', params["cobbler_profile"])
     
        # These values ultimately get used by genome-firstboot
        ksmeta = "fqdn=#{name} " +
                 "genome_repo=#{@fqdn} " + 
                 "cloudhost=#{params["cloudhost"]}"

        ksmeta << " email=#{email}" if email

        cobbler_call('modify_system', system_id, 'ksmeta', ksmeta)

        if params.has_key? "cobbler_virt_ram"
          cobbler_call('modify_system', system_id, 'virt-ram', params["cobbler_virt_ram"])
        end
        if params.has_key? "cobbler_virt_disk"
          cobbler_call('modify_system', system_id, 'virt-file-size', params["cobbler_virt_disk"])
        end
        cobbler_call('save_system', system_id)
      end

      def register_machine(machine_fqdn, config, email="")
        machine_type = config["parameters"]["genome_machine_type"]
        post_yaml(machine_fqdn, machine_type, YAML.dump(config))

        params = config["parameters"]
        # The profile will be nil if the user is using a machine that already has an OS
        add_system_to_cobbler(machine_fqdn, params, email) unless params["cobbler_profile"].nil?
      end

      def get_system_ip(system_name, max_tries=1)
        status = 1.upto(max_tries) do
          # Technically the yield should be after the remote call, but it would
          # trip up the 'find'
          yield if block_given?

          data = cobbler_call('get_status').find do |ip, s|
            s[2] == "system:%s" % system_name && s[5] =~ /^installing/
          end

          # This returns from the block
          break data unless data.nil?
        end

        return status.nil? ? nil : status[0]
      end

      private

      def cobbler_token
        # TODO: figure out a way to pass these values in
        # This method is the only method outside of "cobbler_call" that can
        # directly use the XMLRPC interface to call cobbler.
        @cobbler_token ||= cobblerd.call2("login", "cobbler", "password")[1]
      end

    end

    ## The Cloud class defines a Cloud controller object that
    #  can report status on various Cloudhosts.
    class Cloud
      include Updateable

      # Create a new CloudController with the given server name (prefix or fqdn)
      def initialize(name) 
        @service_url = "http://#{name}/cloud"
      end

      # The URL for the cloud API
      def cloud
        @service_url
      end
    end

    ## The Cloudhost class defines a host that is
    #  part of a larger cloud.  A Cloudhost knows:
    #    * its own cloudmaster (i.e., Cloud object)
    #    * how to provision new machines 
    #    * how to report information to the Cloud 
    #      regarding virt status
    class Cloudhost
      include Updateable

      def initialize(fqdn)
        @fqdn = fqdn 
        @service_url = "http://#{fqdn}/host"
        
        #if not valid?
          #raise InvalidAppliance, "Invalid Cloudhost URL"
        #end
        #
        #
      end

      # The actual URL for this host (with the appropriate URI attached)
      def host
        @service_url 
      end

      # Create a new machine on this host 
      # NOTE:  Restr will swallow error messages under
      # the useless "500 Internal Server Error".
      # A better way to create machines is to simply 
      # execute an HTTP POST to:
      # http://#{fqdn}/host/koan
      # with the following params:
      # system_id (the name of the cobbler system)
      # repo (the fqdn of the cobbler server)
      # email (the VM's owner's email address)
      def create_machine(system_id, email, repo)
        if not repo.is_a? GenomeLib::Appliances::Genome
          raise ArgumentError, "repo parameter must be of type GenomeLib::Appliances::Genome"
        end
        return Restr.post("#{@service_url}/koan", :system_id => system_id, :repo => repo.fqdn, :email => email)
      end
    end
  end
end

