require 'date'
require 'facter'
require 'thread'

module GenomeLib
  class Evictor
    def initialize(model, seconds, logger=$LOG)
      @model = model
      @seconds = seconds.to_i
      @logger = logger
    end

    def run
      cut_off = Time.now - @seconds
      stale_records = @model.find(:all, :conditions => ["last_updated < ?", cut_off])
      stale_records.each {|r| log("Evicting: #{r}")}
      @model.destroy(stale_records)
    end

    def log(msg)
      @logger.debug(msg) unless @logger.nil?
    end
  end

  class HostReporter
    include GenomeLib::Appliances

    def initialize(model)
      @model = model 
    end

    def run
      ip = Facter[:ipaddress].value
      fqdn = Facter[:fqdn].value
      virt_type = if File.exist?("/dev/kvm")
                   "KVM"
                 elsif File.exist?("/proc/xen")
                   "Xen"
                 else
                   "Unknown"
                 end

      virsh = Virsh.new
      ni = virsh.nodeinfo

      guests = @model.find(:all).map do |g|
        VirtGuest.new(g.name, g.uuid, g.hostname, g.ipaddress, g.appliance, g.status, g.arch, g.memsize)
      end

      c = Cloud.new(::Genome[:cloudmaster])
      c.update_machine(::Genome[:uuid], :ipaddress => ip,
                                        :virt_type => virt_type,
                                        :hostname => fqdn,
                                        :arch => Facter[:architecture].value,
                                        :totalmem => ni.memsize,
                                        :freemem => ni.memsize - @model.used_mem,  
                                        :guests => guests.to_yaml)
    end
  end

  class CloudThread
    @@cloudmutex = Mutex.new

    def initialize(runner, freq, name, logger=nil)
      @runner = runner
      @freq = freq
      @name = name
      @logger = logger
    end

    def run
      @thread = Thread.new do
        loop do
          log("Running #{@name} thread") 
          begin
            @@cloudmutex.synchronize {@runner.run}
          rescue
            log($!) 
            log($!.message) 
          end
   
          log("Trying #{@name} thread again in #{@freq} seconds") 
          sleep @freq
        end 
      end
    end

    def stop
      @thread.stop
    end

    private 
    def log(msg)
      @logger.debug(msg) unless @logger.nil?
    end
  end
end
