# Copyright (C) 2008 Red Hat, Inc

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# a long with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# These are required by the Rakefile.  The tests should probably run in a separate process.
$:.unshift(File.dirname(__FILE__) + "/../lib")
require 'genome-lib/virsh'
require 'test/unit'
require 'mocha'

include GenomeLib

class VirshListTestCase < Test::Unit::TestCase
  def setup
    @virsh = Virsh.new
    @test_uuid = `uuidgen`.strip
    @virsh.stubs(:get_uuid).returns(@test_uuid)

    @list_output = [" Id Name                 State", 
                    "----------------------------------", 
                    "  0 Domain-0             running", 
                    "  6 genome-arch          blocked", 
                    "728 bleanhar-test4       no state", 
                    " 35 413bd21d-badc-4227-958b-035f0a36d118 blocked"]
    @list_all_output = @list_output + 
                    ["  - 3956b16a-b04f-4cb3-a5f6-d1669d2e12fd shut off", 
                     "  - b4dc9033-0bb7-4ba0-862e-d06ec4ad80d8 shut off", 
                     "  - ee7955f2-3a1d-4351-a930-f370e4deb3f7 shut off"]
  end 

  def test_list
    @virsh.expects(:run_command).returns(@list_output)
    guests = @virsh.list 
    assert_equal(3, guests.size)

    assert(guests.find {|g| g.name == "genome-arch" && g.running?})
    assert(guests.find {|g| g.name == "bleanhar-test4" && ! g.running?})
    assert(guests.find {|g| g.name == "413bd21d-badc-4227-958b-035f0a36d118" && g.running?})
  end

  def test_list_all
    @virsh.expects(:run_command).returns(@list_all_output)
    guests = @virsh.list(:all => true)
    assert_equal(6, guests.size)

    #assert(guests.find {|g| g.name == "Domain-0" && g.running?})
    assert(guests.find {|g| g.name == "genome-arch" && g.running?})
    assert(guests.find {|g| g.name == "bleanhar-test4" && ! g.running?})
    assert(guests.find {|g| g.name == "413bd21d-badc-4227-958b-035f0a36d118" && g.running?})
    assert(guests.find {|g| g.name == "3956b16a-b04f-4cb3-a5f6-d1669d2e12fd" && ! g.running?})
    assert(guests.find {|g| g.name == "b4dc9033-0bb7-4ba0-862e-d06ec4ad80d8" && ! g.running?})
    assert(guests.find {|g| g.name == "ee7955f2-3a1d-4351-a930-f370e4deb3f7" && ! g.running?})
  end

  def test_uuid_set
    @virsh.expects(:run_command).returns(@list_output)
    guests = @virsh.list 

    assert_equal(@test_uuid, guests.first.uuid)
  end
end

class VirshXMLTestCase < Test::Unit::TestCase
  def setup
    @virsh = Virsh.new

    @guest_xml = %{
      <domain type='xen' id='1'>
        <name>bleanhar-test8</name>
        <uuid>dcc7e431-2d38-9f3f-a768-f264b348d5d4</uuid>
        <bootloader>/usr/bin/pygrub</bootloader>
        <os>
          <type>linux</type>
          <kernel>/var/lib/xen/boot_kernel.Pke4OF</kernel>
          <initrd>/var/lib/xen/boot_ramdisk.H7bwba</initrd>
          <cmdline>ro root=LABEL=/ console=xvc0</cmdline>
        </os>
        <memory>262144</memory>
        <vcpu>1</vcpu>
        <on_poweroff>destroy</on_poweroff>
        <on_reboot>restart</on_reboot>
        <on_crash>restart</on_crash>
        <devices>
          <interface type='bridge'>
            <source bridge='xenbr0'/>
            <target dev='vif1.0'/>
            <mac address='00:16:3e:2e:9f:8c'/>
            <script path='vif-bridge'/>
          </interface>
          <disk type='file' device='disk'>
            <driver name='file'/>
            <source file='/images/bleanhar-test8-disk0'/>
            <target dev='xvda'/>
          </disk>
          <console tty='/dev/pts/0'/>
        </devices>
      </domain>
      }
  end

  def test_get_guest_mem
    @virsh.expects(:run_command_raw).returns(@guest_xml)
    assert_equal(262144, @virsh.guest_mem("bleanhar-test8"))

  end
end

class VirshInfoTestCase < Test::Unit::TestCase
  def setup
    @virsh = Virsh.new

    @nodeinfo_output = ["CPU model:           x86_64",
                        "CPU(s):              4",
                        "CPU frequency:       1861 MHz",
                        "CPU socket(s):       2",
                        "Core(s) per socket:  2",
                        "Thread(s) per core:  1",
                        "NUMA cell(s):        1",
                        "Memory size:         4191232 kB"]
  end

  def test_node_info
    @virsh.expects(:run_command).returns(@nodeinfo_output)
    ni = @virsh.nodeinfo
    assert_equal("x86_64", ni.cpu_model)  
    assert_equal(4191232, ni.memsize)  
  end
end
